/*
 * Created on 08.07.2004
 */
package thegreatestsynthofall;

import javax.swing.*;
import java.awt.event.*;
import javax.sound.sampled.*;

/**
 * @author jl
 */
public class TheGreatestSynthOfAll extends JFrame {

	public static void main(String[] args) {
		TheGreatestSynthOfAll myFrame = new TheGreatestSynthOfAll();	
	}
	
	private TheGreatestSynthOfAll()
	{
		new SynthThread().start();
		getContentPane().add(new JLabel("Use ASD... on the keyboard"));
		getContentPane().addKeyListener(new KeyboardAdapter());
		pack();
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);		
		getContentPane().requestFocusInWindow();
		show();
	}
}

// We rely on the chording capability of most
// modern keyboards. However, they may only
// allow certain combinations of keys.
class KeyboardAdapter implements KeyListener
{
	public void keyPressed(KeyEvent e)
	{
		int k = convKey(e.getKeyChar());
		if(k >= 0)
			SynthThread.keyPressed[k] = true;
	}
	
	public void keyReleased(KeyEvent e)
	{
		int k = convKey(e.getKeyChar());
		if(k >= 0)
			SynthThread.keyPressed[k] = false;
	}
	
	public void keyTyped(KeyEvent e)
	{}
	
	private int convKey(char k)
	{
		switch(k)
		{
			case 'a' : return  0;
			case 'w' : return  1;
			case 's' : return  2;
			case 'e' : return  3;
			case 'd' : return  4;
			case 'f' : return  5;
			case 't' : return  6;
			case 'g' : return  7;
			case 'z' : return  8; // 'y' in Italy
			case 'h' : return  9;
			case 'u' : return 10;
			case 'j' : return 11;
			case 'k' : return 12;			
			case 'o' : return 13;
			case 'l' : return 14;
			case 'p' : return 15;
			default: return -1;
		}
	}
}

class SynthThread extends Thread
{
	final static int numKeys = 16;
	static volatile boolean[] keyPressed = new boolean[numKeys];
	
	public void run()
	{	
		final float samplingRate = 44100.0f;
		final int blockSize = 2048; // number of samples
		byte[] bufferOut = new byte[2*blockSize]; // 16 bit mono

		// oscillators
		double[] increment = new double[numKeys];
		for(int k = 0; k < numKeys; k++)
		{
			increment[k] = 440.0*Math.pow(2.0, (k-9)/12.0)/samplingRate;
		}
		double[] oscillatorPhase = new double[numKeys];
		double[] envelope = new double[numKeys];
		
		try
		{
			AudioFormat af = new AudioFormat(samplingRate, 16, 1, true, false);
			DataLine.Info lineInfo = new DataLine.Info(SourceDataLine.class, af);
			SourceDataLine myLine = (SourceDataLine)AudioSystem.getLine(lineInfo);
			myLine.open(af, 2048*3);		
			myLine.start();
			
			// ugly: thread is killed, not stopped in an orderly fashion
			while(true)
			{
				for(int i = 0; i < blockSize; i++)
				{
					double out = 0.0;
					
					// expensive: full polyphony
					for(int k = 0; k < numKeys; k++)
					{
						// a simple AR-type envelope
						if(keyPressed[k])
							envelope[k] = 0.999*envelope[k] + 0.001;
						else
							envelope[k] = 0.9999*envelope[k];
							
						// modulating the pitch by the envelope
						oscillatorPhase[k] += (0.8+0.2*envelope[k])*increment[k];
						if(oscillatorPhase[k] > 1.0)
							oscillatorPhase[k] -= 1.0;
						
						out += 0.1*envelope[k]*Math.sin(2*Math.PI*oscillatorPhase[k]);
					}
					
					short w = (short)(out*Short.MAX_VALUE);
					bufferOut[2*i  ] = (byte)(w & 0xFF);
					bufferOut[2*i+1] = (byte)(w >> 8);
				}
				myLine.write(bufferOut, 0, bufferOut.length);
			}
		}
		catch(Exception e)
		{
			throw new RuntimeException(e);
		}		
	}
}